/*
** Module   : ControlSetConfig.java
** Abstract : stores common state for widgets with items set
**
** Copyright (c) 2005-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 EVL 20050714   @21692 Creation based on general concept. Necessary modifications to get
**                           compiled.
** 002 EVL 20050724   @21804 Changing the default delimiter to comma char.
** 003 EVL 20050728   @21909 Moving screenValue variable to ControlConfig from ControlSetConfig
**                           because not only set like widgets must use it, but text related too
**                           (e.g. FillIn).
** 004 SIY 20050831   @22496 Refactoring (see @@22394-22409)
** 005 SIY 20050907   @22628 Fixed NullPointerException.
** 006 SIY 20051129   @23560 Added addFirst()/addLast() methods.
** 007 NVS 20051129   @23565 Class access changed to public 
** 008 SIY 20060209   @24478 Major refactoring.
** 009 SIY 20060210   @24524 Added getSelection() method, switched to ControlSetComponent
**                           interface.
** 010 NVS 20060313   @25041 Moved format, setFormat() and getFormat() from the subclass,
**                           ComboBoxConfig, so that format methods are available in
**                           SelectionList as well.
** 011 GES 20060803   @28402 Made externalizable for better performance. Many code standard
**                           cleanups.
** 012 SIY 20061102   @30941 Minor changes caused by field encapsulation.
** 013 GES 20061109   @31052 Removed dead code and ensured that the format string gets copied.
** 014 SIY 20061213   @31673 honorFormat now defaults to true. Added pairs flag.
** 015 SIY 20100619          Switching to own implementation of character UI.
** 016 SIY 20100804          Changes in imports, some refactorings.
** 017 SIY 20110112          Changes related to new color handling scheme.
** 018 SIY 20111101          Moved from CHUI package. Minor cleanups/refactorings to prepare for
**                           GUI client.
** 019 GES 20140627          Changed color attribute method names to match naming standards.
** 020 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 021 IAS 20150202          Added item id generator
** 022 IAS 20150305          Changed 'pairs' type to Boolean
** 023 IAS 20150312          Added 'pairsSetWhenRealized' field
*/

package com.goldencode.p2j.ui;

import java.io.*;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * This class holds the common attributes for all widgets which have a set
 * of the control items inside.
 */
public class ControlSetConfig
extends ControlConfig
{
   /** The character value to separate the items inside the item string. */
   public char delimiter = ',';

   /** <code>true</code> to force items to be sorted. */
   public boolean sort = false;

   /** The string representing the inner data format for the items. */
   public String format = null;

   /** Database name. */
   public String dbname = null;

   /** Table name. */
   public String table = null;
   
   /** Array of items */
   public ControlSetItem[] items = new ControlSetItem[0];

   /** Tell widget that format should be honored. */
   public boolean honorFormat = true;
 
   /** Value of the "realized' flag when items' type (pairs/labels) was set*/
   public boolean itemsSetWhenRealized = false;

   /** Set to true if items defined as pairs */
   public Boolean pairs = null;
   
   /** item id generator */
   public int itemId = 1;
   
   /** The height of the list widget. */
   public double listHeightChars = 0;
   
   /** The width of the list widget. */
   public double listWidthChars = 0;

   /**
    * Default constructor (only used in de-serialization).
    */
   public ControlSetConfig()
   {
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected ControlSetConfig(int id)
   {
      super(id);
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected ControlSetConfig(WidgetId id)
   {
      super(id);
   }

   /**
    * Synchronize the configuration with the widget state.
    */
   @Override
   public void syncWithWidget()
   {
      super.syncWithWidget();
      
      if (id == null || !onClientSide())
      {
         // nothing for the server-side
         return;
      }

      Widget<?> instance = tc.getWidget(id);
      
      if (instance == null)
      {
         return;
      }

      ((ControlSetComponent) instance).refreshItems();
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      ControlSetConfig cfg = (ControlSetConfig) config;
      
      delimiter = cfg.delimiter;
      sort = cfg.sort;
      format = cfg.format;
      dbname = cfg.dbname;
      table = cfg.table;
      items = cfg.items;
      honorFormat = cfg.honorFormat;
      pairs = cfg.pairs;
      itemsSetWhenRealized = cfg.itemsSetWhenRealized;
      heightChars = cfg.heightChars;
      listHeightChars = cfg.listHeightChars;
      listWidthChars = cfg.listWidthChars;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      delimiter   = in.readChar();
      sort        = in.readBoolean();
      honorFormat = in.readBoolean();
      pairs       = (Boolean)in.readObject();
      itemsSetWhenRealized = in.readBoolean();
      format      = (String) in.readObject();
      items       = (ControlSetItem[]) in.readObject();
      table       = (String) in.readObject();
      dbname      = (String) in.readObject();
      itemId      = in.readInt();
      listHeightChars = in.readDouble();
      listWidthChars  = in.readDouble();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeChar(delimiter);
      out.writeBoolean(sort);
      out.writeBoolean(honorFormat);
      out.writeObject(pairs);
      out.writeBoolean(itemsSetWhenRealized);
      out.writeObject(format);
      out.writeObject(items);
      out.writeObject(table);
      out.writeObject(dbname);
      out.writeInt(itemId);
      out.writeDouble(listHeightChars);
      out.writeDouble(listWidthChars);
   }
   
   /**
    * Initialize this config.
    */
   protected void init()
   {
      pfcolor = Color.MESSAGES;
   }
   
}
